-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: PlayerHPS")

L:RegisterTranslations("enUS", function() return {
	["Healing per Second"] = true,

	["Track HPS"] = true,
	["Whether or not to track Healing per Second."] = true,

	["HPS: %d (#%s)."] = true,
	["HPS: "] = true,

	["HPS"] = true,
	["Player"] = true,
	["#"] = true,
} end)

-----------------------------------------------------------------------
-- Local heap
-----------------------------------------------------------------------

local new, del
do
	local cache = setmetatable({},{__mode='k'})
	function new(...)
		local t = next(cache)
		if t then
			cache[t] = nil
			for i = 1, select("#", ...) do
				table.insert(t, (select(i, ...)))
			end
			return t
		else
			return {...}
		end
	end
	function del(t)
		if type(t) == "table" then
			for k, v in pairs(t) do
				if type(v) == "table" then
					del(v)
				end
				t[k] = nil
			end
			cache[t] = true
		end
		return nil
	end
end

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local tablet = AceLibrary("Tablet-2.0")

local percentString = "(%d%%)"

local playerName = nil

local event = {
	eventType = "Heal",
	sourceID = "player",
}

local data = nil
local position = nil
local dirty = 0
local lastUpdateTime
local seconds = nil
local oldHPS = 0

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule("PlayerHPS", "Parser-3.0")
mod.revision = tonumber(("$Revision: 28702 $"):sub(12, -3)) or 1

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
	playerName = UnitName("player")
end

function mod:OnModuleEnable()
	self:RegisterParserEvent(event, "OnParserEvent")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
end

function mod:OnReset()
	if type(data) == "table" then
		data = del(data)
	end

	data = new()
	data[playerName] = 0
	seconds = 0
	oldHPS = 0
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return L["Healing per Second"] end
function mod:GetBroadcastData() return dirty end

function mod:IsDirty()
	return dirty ~= 0
end

function mod:OnRemoteData(sender, input)
	-- remote HPS is placed in the damage table, this is done so we don't need to recalc everybody else's HPS.
	data[sender] = (data[sender] or 0) + input
end

function mod:PLAYER_REGEN_ENABLED()
	lastUpdateTime = nil
end

function mod:OnParserEvent(event)
	assert(event.sourceID == "player", "Not supposed to get combat events from people except ourselves.")
	if event.amount then
		local newTime = GetTime()
		local timeSinceLastUpdate
		-- on the offhand chance that lastUpdateTime is nil at this point (if somehow we do damage before a regen disabled event fires)
		-- make sure that we have a 
		if not lastUpdateTime then
			timeSinceLastUpdate = 0.1
		else
			timeSinceLastUpdate = newTime - lastUpdateTime
		end
		lastUpdateTime = newTime
		data[playerName] = data[playerName] + event.amount
		seconds = seconds + timeSinceLastUpdate
		local newHPS = data[playerName] / seconds
		dirty = newHPS - oldHPS
		oldHPS = newHPS
	end
end

local function Sorter(a, b) return a[2] > b[2] end
function mod:OnTooltipUpdate()
	if not data then return end

	local tbl = new()
	for k, v in pairs(data) do
		if type(v) == "number" and v > 0 then
			if k == playerName then
				table.insert(tbl, new(k, v/seconds))
			else
				table.insert(tbl, new(k, v))
			end
		end
	end

	if #tbl > 0 then
		local cat = tablet:AddCategory(
			"columns", #tbl > 1 and 4 or 3,
			"text", L["#"],
			"text2", L["Player"],
			"text3", L["HPS"],
			"child_justify", "LEFT",
			"child_justify2", "LEFT",
			"child_justify3", "RIGHT",
			"child_justify4", "RIGHT",
			"child_textR", 1,
			"child_textG", 1,
			"child_textB", 0,
			"child_text3R", 1,
			"child_text3G", 1,
			"child_text3B", 1,
			"child_text4R", 0.65,
			"child_text4G", 0.65,
			"child_text4B", 0.65
		)

		table.sort(tbl, Sorter)

		local percentage = nil
		if #tbl > 1 then
			local total = 0
			for i, v in ipairs(tbl) do
				total = total + v[2]
			end
			percentage = new()
			for i, v in ipairs(tbl) do
				table.insert(percentage, math.floor((v[2] * 100) / total))
			end
		end

		local playerShown = nil
		for i, v in ipairs(tbl) do
			if i > 10 then break end -- Only show the top 10.
			if percentage then
				cat:AddLine(
					"text", tostring(i) .. ".",
					"text2", self:GetColoredName(v[1]),
					"text3", math.floor(v[2]),
					"text4", string.format(percentString, percentage[i])
				)
			else
				cat:AddLine(
					"text", tostring(i) .. ".",
					"text2", self:GetColoredName(v[1]),
					"text3", math.floor(v[2])
				)
			end
			if v[1] == playerName then
				playerShown = true
				if #tbl > 1 then
					position = i
				else
					position = nil
				end
			end
		end

		percentage = del(percentage)

		if not playerShown and data[playerName] > 0 then
			cat:AddLine("text", "?", "text2", self:GetColoredName(playerName), "text3", tostring(data[playerName]/seconds))
			position = "?"
		end
	end

	-- Cleanup.
	tbl = del(tbl)
end

function mod:GetDisplayText()
	if data and data[playerName] > 0 then
		return L["HPS: "] .. (type(position) ~= "nil" and tostring(position) .. ". " or "") .. math.floor(data[playerName]/seconds)
	end
	return nil
end

